import { type LanguageServiceContext } from '@volar/language-service';
import type * as html from 'vscode-html-languageservice';
import type { PugDocument } from '../pugDocument';
export declare function register(htmlLs: html.LanguageService): (pugDoc: PugDocument, pos: html.Position, serviceContext: LanguageServiceContext, documentContext?: html.DocumentContext, options?: html.CompletionConfiguration) => Promise<html.CompletionList | undefined>;
//# sourceMappingURL=completion.d.ts.map