import type { LanguageServicePlugin } from '@volar/language-service';
export declare function create({ getComponentProps }: import('@vue/typescript-plugin/lib/requests').Requests): LanguageServicePlugin;
