import { type Language, type SourceScript, type VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
export interface ComponentPropInfo {
    name: string;
    description?: string;
    optional?: boolean;
    boolean?: boolean;
}
export declare function getComponentProps(ts: typeof import('typescript'), tsLanguageService: ts.LanguageService, program: ts.Program, language: Language, sourceScript: SourceScript<string>, virtualCode: VueVirtualCode, position: number): ComponentPropInfo[];
