import type { Code } from '../../types';
import type { StyleCodegenOptions } from '.';
export declare function generateStyleScopedClasses({ vueCompilerOptions, styles }: StyleCodegenOptions): Generator<Code>;
