import type { VueCodeInformation } from '../types';
export declare const codeFeatures: {
    full: VueCodeInformation;
    all: VueCodeInformation;
    importCompletionOnly: VueCodeInformation;
    verification: VueCodeInformation;
    completion: VueCodeInformation;
    withoutCompletion: VueCodeInformation;
    navigation: VueCodeInformation;
    navigationWithoutRename: VueCodeInformation;
    navigationAndCompletion: VueCodeInformation;
    navigationAndVerification: VueCodeInformation;
    withoutNavigation: VueCodeInformation;
    semanticWithoutHighlight: VueCodeInformation;
    withoutHighlight: VueCodeInformation;
    withoutHighlightAndCompletion: VueCodeInformation;
    withoutSemantic: VueCodeInformation;
    structure: VueCodeInformation;
    structureAndFormat: VueCodeInformation;
    format: VueCodeInformation;
    doNotReportTs2339AndTs2551: VueCodeInformation;
    doNotReportTs2353AndTs2561: VueCodeInformation;
    doNotReportTs6133: VueCodeInformation;
};
/** @deprecated use `codeFeatures.full` instead */
export declare const allCodeFeatures: VueCodeInformation;
