import type { LanguageServiceContext, SourceScript } from '@volar/language-service';
import type { VueVirtualCode } from '@vue/language-core';
import { URI } from 'vscode-uri';
export declare function resolveEmbeddedCode(context: LanguageServiceContext, uriStr: string): {
    script: Required<SourceScript<URI>>;
    code: import("@volar/language-service").VirtualCode;
    root: VueVirtualCode;
} | undefined;
