import type { LanguageServiceContext } from '@volar/language-service';
import type { URI } from 'vscode-uri';
export declare const enum TagNameCasing {
    Kebab = 0,
    Pascal = 1
}
export declare const enum AttrNameCasing {
    Kebab = 0,
    Camel = 1
}
export declare function getTagNameCasing(context: LanguageServiceContext, uri: URI): Promise<TagNameCasing | undefined>;
export declare function getAttrNameCasing(context: LanguageServiceContext, uri: URI): Promise<AttrNameCasing | undefined>;
