import type { SFCParseResult } from '@vue/compiler-sfc';
declare module '@vue/compiler-sfc' {
    interface SFCDescriptor {
        comments: string[];
    }
}
export declare function parse(source: string): SFCParseResult;
