import type * as vscode from '@volar/language-service';
export declare function create({ mappedLanguages, }?: {
    mappedLanguages?: Record<string, string>;
}): vscode.LanguageServicePlugin;
//# sourceMappingURL=index.d.ts.map