import type { LanguageServicePlugin } from '@volar/language-service';
import { type IR } from '@vue/language-core';
import type * as ts from 'typescript';
export declare function create(ts: typeof import('typescript'), { collectExtractProps }: import('@vue/typescript-plugin/lib/requests').Requests): LanguageServicePlugin;
export declare function getLastImportNode(ts: typeof import('typescript'), sourceFile: ts.SourceFile): ts.Node | undefined;
export declare function createAddComponentToOptionEdit(ts: typeof import('typescript'), ir: IR, ast: ts.SourceFile, componentName: string): {
    range: import("@vue/language-core").TextRange<ts.ObjectLiteralExpression>;
    newText: string;
} | undefined;
