import type * as ts from 'typescript';
export declare function resolveModuleName(ts: typeof import('typescript'), languageServiceHost: ts.LanguageServiceHost, fileName: string, moduleName: string, allowNonExistent?: boolean): string | undefined;
