import { type Language, type SourceScript, type VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
export declare function isRefAtPosition(ts: typeof import('typescript'), language: Language, program: ts.Program, sourceScript: SourceScript<string>, virtualCode: VueVirtualCode, position: number): boolean;
