import type { LanguageServicePlugin } from '@volar/language-service';
import type * as ts from 'typescript';
export declare function create(ts: typeof import('typescript')): LanguageServicePlugin;
/**
 * Refactored from https://github.com/vuejs/core/blob/main/packages/compiler-sfc/src/script/definePropsDestructure.ts
 */
export declare function findDestructuredProps(ts: typeof import('typescript'), ast: ts.SourceFile, props: MapIterator<string>): [ts.Identifier, boolean][];
