import type * as html from 'vscode-html-languageservice';
import type { PugDocument } from '../pugDocument';
export declare function register(htmlLs: html.LanguageService): (pugDoc: PugDocument, pos: html.Position, options?: html.CompletionConfiguration) => string | null | undefined;
//# sourceMappingURL=quoteComplete.d.ts.map