import type { Language, SourceScript, VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
import type { ComponentMeta } from 'vue-component-meta';
export declare function getComponentMeta(ts: typeof import('typescript'), program: ts.Program, language: Language, getSourceScript: (fileName: string) => SourceScript | undefined, sourceFile: ts.SourceFile, virtualCode: VueVirtualCode, tag: string): ComponentMeta | undefined;
