import type { LanguageServicePlugin } from '@volar/language-service';
/**
 * A language service plugin that provides completion for Vue directive comments,
 * e.g. if user is writing `<!-- |` in they'll be provided completions for `@vue-expect-error`, `@vue-generic`, etc.
 */
export declare function create(): LanguageServicePlugin;
