import { type LanguageServicePlugin } from '@volar/language-service';
export declare function create(ts: typeof import('typescript'), languageId: 'html' | 'jade', tsserver: import('@vue/typescript-plugin/lib/requests').Requests): LanguageServicePlugin;
