import type * as ts from 'typescript';
import type { ComponentPropInfo } from './getComponentProps';
export declare function getElementAttrs(ts: typeof import('typescript'), program: ts.Program, fileName: string, tag: string): ComponentPropInfo[];
