"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLength = void 0;
function getLength(segments) {
    let length = 0;
    for (const segment of segments) {
        length += typeof segment == 'string' ? segment.length : segment[0].length;
    }
    return length;
}
exports.getLength = getLength;
//# sourceMappingURL=getLength.js.map