import type { LanguageServicePlugin } from '@volar/language-service';
import { create as createHtmlService } from 'volar-service-html';
import type { TextDocument } from 'vscode-languageserver-textdocument';
import * as pug from './lib/languageService';
export interface Provide {
    'pug/pugDocument': (document: TextDocument) => pug.PugDocument | undefined;
    'pug/languageService': () => pug.LanguageService;
}
export declare function create({ documentSelector, configurationSections, ...options }?: Parameters<typeof createHtmlService>[0]): LanguageServicePlugin;
//# sourceMappingURL=index.d.ts.map